# BluePrint

A library to decouple rendering from logic in a React application

## Installation

```
npm install o-blue-print
```

## Documentation

[http://o-programming-language.org/](http://o-programming-language.org/)

## Usage

The concept of BluePrint is quite simple.

The first half is to give to all React Components with application logic (as opossed to rendering logic alone) a blue print with the specifications of what it should render and let the Component render it instead of passing around many different props.

The other half is to move all the logic not strictly related to the rendering phase out from the Component to its BluePrint.

With this approach no component ever calls `.setState({...})` to change the application state. Instead the component calls named methods on its BluePrint and it's the BluePrint the one that invalidates the current state to produce a new rendering phase.

This has a huge impact in the application tests since all there's left in a component to test is that

- given a BluePrint with a fixed set of values it renders the expected contents
- each event calls the proper method in the blue print.

The application logic is tested on the BluePrint side alone without any rendering or events involved. It's just regular objects and methods.

# Tutorial

Before going into the details of the BluePrint library classes and methods I will implement a React application using BluePrint.

The application is the same as the one in the [React tutorial](https://reactjs.org/tutorial/tutorial.html). 

For this tutorial I will follow the [Create React App](https://github.com/facebook/create-react-app) setup for the sake of simplicity but the integration is the same for any other setup of React.

You can find the final source code of this tutorial in the [tic-tac-toe](./examples/tic-tac-toe) directory.


As a suggestion to walk through this tutorial make the modifications on each step and run the application in a real environment immediatly after.

Most of the times the application will fail with an error.

That's ok. That is what it's supposed to happen.

Design the application top down. Write how would you like to use the application before writing the implementation.

That leads to simplier, cleaner interfaces for your objects.

And use the error raised by the application as a suggestion from her telling you what method you should implement next.

Let the application guide you while you are building it.

## 1 - Add the blue print dependency

Add `o-blue-print` to the project.

```
npm install o-blue-print
```

## 2 - Create the root component

In this case the component is already created in the tutorial base code and looks like this

```javascript
// src/index.js
import React from 'react'
import ReactDOM from 'react-dom'
import GameComponent from './components/GameComponent'
import './index.css'

const gameComponent = (
  <React.StrictMode>
    <GameComponent />
  </React.StrictMode>
)

ReactDOM.render(gameComponent, document.getElementById('root'))
```

I will also create a `GameComponent` placeholder

```javascript
// src/components/GameComponent.js
import React from 'react'
import ReactDOM from 'react-dom'

class GameComponent extends React.Component {
  render () {
    const app = this.props.app
    return (
      <div className="game">
        tic-tac-toe
      </div>
    )
  }
}

export default GameComponent
```

Now the app is up and running.

## 3 - Connect the root component to the BluePrint main component

To connect React with BluePrint I will create the main blue print of the application and pass it as the only prop to the `<GameComponent />`

As a covention in this tutorial all blue print objects are passed in the `app={}` prop of the components.

It proved to be easier to read and more clear than naming it `bluePrint` but you can name
the prop following your own convention.


```javascript
// src/index.js
import React from 'react'
import ReactDOM from 'react-dom'
import GameComponent from './components/GameComponent'
import Game from './bluePrints/Game'  // <-- GameComponent blue print
import './index.css'

// Create the MainBluePrint
const game = new Game()       // <-- Create a Game object

const gameComponent = (
  <React.StrictMode>
    <GameComponent app={game} />    // <-- Give the GameComponent its blue print
  </React.StrictMode>
)

ReactDOM.render(gameComponent, document.getElementById('root'))
```

I will also create the Game placeholder

```javascript
// src/bluePrints/Game.js

class Game {

}

export default Game
```

## 4 - Connect <GameComponent> to its Game blue print

A key point of BluePrint design is to decouple the application logic from the render logic.

For that reason BluePrints are unaware of Components yet they are responsible for asking to re-render when something changes on their side.

In order to keep the Component tree in sync with its BluePrint the `Observer pattern` is used. [Note 1]

The Game blue print is the MainBluePrint of this application and emits the event `invalidate` whenever something changes.

The `<GameComponent>` listens to `invalidate` events and triggers a re-render on itself.

I need to connect `<GameComponent />` with its `Game` blue print events:

```javascript
// src/components/GameComponent.js
import React from 'react'
import ReactDOM from 'react-dom'
import BoardComponent from './BoardComponent'

class GameComponent extends React.Component {
  constructor(props) {
    super(props)
    this.invalidateComponent = this.invalidateComponent.bind(this)    
  }

  componentDidMount() {
    const app = this.props.app
    app.on('invalidate', this.invalidateComponent)
  }

  componentWillUnmount() {
    const app = this.props.app
    app.removeListener('invalidate', this.invalidateComponent)
  }

  invalidateComponent() {
    this.forceUpdate()
  }

  render () {
    return (
      <div className="game">
        <div className="game-board">
          <BoardComponent />
        </div>
        <div className="game-info">
          <div>{/* status */}</div>
          <ol>{/* TODO */}</ol>
        </div>
      </div>
    )
  }
}

export default GameComponent
```

and I will create a `BoardComponent` placeholder

```javascript
// src/components/BoardComponent.js
import React from 'react'
import ReactDOM from 'react-dom'

class BoardComponent extends React.Component {
  render () {
    return (
      <div>
      </div>
    )
  }
}

export default BoardComponent
```

## 5 - Create the Game main blue print

An application usually has many blue prints but only one `MainBluePrint`.

The `MainBluePrint` is the root of the application tree and the one that triggers an `invalidate` event to re-render the tree.

It also has other responsibilities like keeping track of the app global state, interfacing with the document DOM and keeping the route path in sync with the state of the app but for the time being I will ignore all of that.

I will create a `Game` class that extends from `MainBluePrint` in its simpliest form

```javascript
// src/bluePrints/Game.js
import {MainBluePrint} from 'o-blue-print'

class Game extends MainBluePrint {
}

export default Game
```

## 6 - Move the logic out from the `<GameComponent>` into the Game blue print

Currently the `<GameComponent>` renders a child `<BoardComponent>`.

The `<BoardComponent>` renders which player turn is and the board squares.

That information is kept by a `Board` blue print (that does not exist yet) and is passed from the `<GameComponent>` to the `<BoardComponent>`

```javascript
// src/components/GameComponent.js
import React from 'react'
import ReactDOM from 'react-dom'
import BoardComponent from './BoardComponent'

class GameComponent extends React.Component {
  constructor(props) {
    super(props)
    this.invalidateComponent = this.invalidateComponent.bind(this)    
  }

  componentDidMount() {
    const app = this.props.app
    app.on('invalidate', this.invalidateComponent)
  }

  componentWillUnmount() {
    const app = this.props.app
    app.removeListener('invalidate', this.invalidateComponent)
  }

  invalidateComponent() {
    this.forceUpdate()
  }

  render () {
    const app = this.props.app
    return (
      <div className="game">
        <div className="game-board">
          <BoardComponent app={ app.getBoard() } />
        </div>
        <div className="game-info">
          <div>{/* status */}</div>
          <ol>{/* TODO */}</ol>
        </div>
      </div>
    )
  }
}

export default GameComponent
```

## 7 - Define the method Game.getBoard()

The previous code complains that `app.getBoard()` is not defined. I will define it

```javascript
// src/bluePrints/Game.js
import {MainBluePrint} from 'o-blue-print'
import Board from './Board'

class Game extends MainBluePrint {
  initialize() {
    this.allowAllMethodsAsCommands()

    this.board = new Board({ parent: this })
  }

  getBoard() {
    return this.board
  }
}

export default Game
```

and create the `Board` placeholder

```javascript
// src/bluePrints/Board.js
class Board {
}

export default Board
```

Here I have a few things to explain.

### initialize()

The first one is the use of the method `initialize()` instead of `constructor()`.

Except for a few rare cases `BluePrint` and `MainBluePrint` classes are initialized in its `initialize` method.

The reason is to ensure that the basic initialization required by the library was already executed and that the blue print object is in a usable and stable state by its initialization time.

Another reason is that overriding `constructor` proved to be error prone when trying to call super with the right parameters.

### this.allowAllMethodsAsCommands()

The second thing to notice is the call to `this.allowAllMethodsAsCommands()`.

By default the methods that a BluePrint class defines are not accesible for other objects to call them. Not even to the component rendering it.

Each BluePrint class decides which methods to expose. To do that it can define a list with

```javascript
this.allowCommands(['method1, method2', ...])
```

or allow all of its methods with the shortcut

```javascript
this.allowAllMethodsAsCommands()
```

Whilst using `allowAllMethodsAsCommands` is less safe than cherry picking each method to expose it's really handy during the prototyping and the exploratory phase of the app.

The important thing to remember is that if a method is not exposed by its blue print it can't be called from any other object other than itself. That is achieved through an (almost) transparent use of Proxy objects.

### new BluePrint({ parent: this })

Lastly to create the child blue print Board the parent is passing itself as a parameter. That parameter is required for child blue prints.

The parent is given for the children to be able to bubble up commands up to the first blue print capable answering it.

I will go through a concrete example of use later in this tutorial to show why a child blue print would like to bubble up commands.

This tutorial does not cover it but it's quite possible for a parent blue print to decide what methods of its own to expose to its children.

## 8 - Make <BoardComponent> to query its Board blue print

Now that the `<BoardComponent>` receives a BluePrint it can query the information to render from it

```javascript
// src/components/BoardComponent.js
import React from 'react'
import ReactDOM from 'react-dom'
import SquareComponent from './SquareComponent'

class BoardComponent extends React.Component {
  render () {
    const app = this.props.app

    return (
      <div>
        <div className="status">{ app.getStatus() }</div>
        <div className="board-row">
          <SquareComponent app={ app.getSquareAtIndex(0) } />
          <SquareComponent app={ app.getSquareAtIndex(1) } />
          <SquareComponent app={ app.getSquareAtIndex(2) } />
        </div>
        <div className="board-row">
          <SquareComponent app={ app.getSquareAtIndex(3) } />
          <SquareComponent app={ app.getSquareAtIndex(4) } />
          <SquareComponent app={ app.getSquareAtIndex(5) } />
        </div>
        <div className="board-row">
          <SquareComponent app={ app.getSquareAtIndex(6) } />
          <SquareComponent app={ app.getSquareAtIndex(7) } />
          <SquareComponent app={ app.getSquareAtIndex(8) } />
        </div>
      </div>
    )
  }
}

export default BoardComponent
```

```javascript
// src/components/SquareComponent.js
import React from 'react'
import ReactDOM from 'react-dom'

class SquareComponent extends React.Component {
  render () {
    const app = this.props.app

    return (
      <div>
      </div>
    )
  }
}

export default SquareComponent
```

## 9 - Create the Board blue print

`Board` blue print is similar to the previous `Game` blue print except that the class extends from `BluePrint`.


```javascript
// src/bluePrints/Board.js
import {BluePrint} from 'o-blue-print'
import Square from './Square'

class Board extends BluePrint {
  initialize() {
    this.allowAllMethodsAsCommands()

    this.squares = [0,1,2,3,4,5,6,7,8].map((i)=> 
      new Square({ parent: this, id: i }))
  }

  getRenderMethodName() {
    return 'renderBoard'
  }

  getStatus() {
    return 'Next player: x'
  }

  getSquareAtIndex(index) {
    return this.squares[index]
  }
}

export default Board
```

## 10 - Create the Square blue print

Pretty much the same than the previous step.

I will create a class `Square` extending `BluePrint`

```javascript
// src/bluePrints/Square.js
import {BluePrint} from 'o-blue-print'

class Square extends BluePrint {
  initialize({id}) {
    this.allowAllMethodsAsCommands()

    this.id = id
  }

  getContent() {
    return this.id
  }
}

export default Square
```

## 11 - Make `<SquareComponent>` to query its blue print

Make `SquareComponent` to query information from its `Square` blue print

```javascript
// src/components/SquareComponent.js
import React from 'react'
import ReactDOM from 'react-dom'

class SquareComponent extends React.Component {
  render () {
    const app = this.props.app
    return (
      <button className="square">
        { app.getContent() }
      </button>
    )
  }
}

export default SquareComponent
```

## 12 - Make the click on a square to fill its content

Now it gets interesting.

So far the Components displayed read only content.

I will modify the `<SquareComponent />` to fill it with an `x` when a user clicks on it

```javascript
// src/components/SquareComponent.js
import React from 'react'
import ReactDOM from 'react-dom'

class SquareComponent extends React.Component {
  render () {
    const app = this.props.app
    return (
      <button className="square" onClick={()=>app.playerMove()}>
        { app.getContent() }
      </button>
    )
  }
}

export default SquareComponent
```

The component only adds a method call to its blue print.

All the logic is delegated to the blue print to make the Component as simple as possible.

For now I will just fill the square with an `x` character. Later I will modify it to make the proper player move.

```javascript
// src/bluePrints/Square.js
import {BluePrint} from 'o-blue-print'

class Square extends BluePrint {
  initialize({id}) {
    this.allowAllMethodsAsCommands()

    this.id = id
    this.content = null
  }

  getId() {
    return this.id
  }

  getContent() {
    return this.content
  }

  playerMove() {
    if (this.content !== null) { return }
    this.content = 'X'
    this.invalidate()
  }
}

export default Square
```

I added a `playerMove` method that checks if the square has a previous move and if it doesn't it sets the new move and invalidates the blue print.

An invalidated blue print means that from the moment of the invalidation the blue print is out of sync with the component that renders it.

The call to `this.invalidate()` bubbles up the invalidation notification up to the top most `Game`, the `MainBluePrint`.

The `MainBluePrint` emits an `invalidate` event and the `<MainComponent>` re-renders itself.

However all a blue print needs to do is to call `this.invalidate()`.

## 13 - Switch turns between players

After the current player makes a move the next turn must be set to the other player.

I will modify 2 blue prints for that, `Square` and `Game`.

I will start with the one that triggers the turn switch, `Square`

```javascript
// src/bluePrints/Square.js
import {BluePrint} from 'o-blue-print'

class Square extends BluePrint {
  initialize({id}) {
    this.allowAllMethodsAsCommands()

    this.id = id
    this.content = null
  }

  getId() {
    return this.id
  }

  getContent() {
    return this.content
  }

  playerMove() {
    if (this.content !== null) { return }
    const currentPlayer = this.parent.getNextPlayer()
    this.content = currentPlayer
    this.parent.shiftPlayer()
    this.invalidate()
  }
}

export default Square
```

I modified the `playerMove` method to ask the `Game` for the next player to move and fill the square with that move.

I also made the method to tell the `Game` to switch the turn afterwards.

The implementation of `Game` changes to


```javascript
// src/bluePrints/Game.js
import {MainBluePrint} from 'o-blue-print'
import Board from './Board'

class Game extends MainBluePrint {
  initialize() {
    this.allowAllMethodsAsCommands()

    this.board = new Board({ parent: this })
    this.currentPlayer = 'X'
  }

  getBoard() {
    return this.board
  }

  getNextPlayer() {
    return this.currentPlayer
  }

  shiftPlayer() {
    this.currentPlayer = this.getNextPlayer() === 'X' ? 'O' : 'X'
  }
}

export default Game
```

Finally I will make the `Board` blue print to update its status too

```javascript
// src/bluePrints/Board.js
import {BluePrint} from 'o-blue-print'
import Square from './Square'

class Board extends BluePrint {
  initialize() {
    this.allowAllMethodsAsCommands()

    this.squares = [0,1,2,3,4,5,6,7,8].map((i)=> 
      new Square({ parent: this, id: i }))
  }

  getRenderMethodName() {
    return 'renderBoard'
  }

  getStatus() {
    return `Next player: ${this.parent.getNextPlayer()}`
  }

  getSquareAtIndex(index) {
    return this.squares[index]
  }
}

export default Board
```

## 14 - Check for the end of the game

After each move I will check if the game finished with a winner or draws using the same function found in `React`s tutorial

```javascript
// src/bluePrints/Game.js
import {MainBluePrint} from 'o-blue-print'
import Board from './Board'

class Game extends MainBluePrint {
  initialize() {
    this.allowAllMethodsAsCommands()

    this.board = new Board({ parent: this })
    this.currentPlayer = 'X'
    this.gameStatus = { finished: false, winner: null }
  }

  getBoard() {
    return this.board
  }

  getNextPlayer() {
    return this.currentPlayer
  }

  shiftPlayer() {
    this.currentPlayer = this.getNextPlayer() === 'X' ? 'O' : 'X'
    this.updateGameStatus()
  }

  updateGameStatus() {
    const winner = this.getWinner()
    if (winner) {
      return this.endWithWinner(winner)
    }
    if (this.noMoreMovesLeft()) {
      return this.endWithDraw()
    }
  }

  noMoreMovesLeft() {
    return this.board.noMoreMovesLeft()
  }

  endWithDraw() {
    this.endWithWinner(null)
  }

  endWithWinner(winner) {
    this.gameStatus = { finished: true, winner }
    return
  }

  getWinner() {
    const lines = [
      [0, 1, 2],
      [3, 4, 5],
      [6, 7, 8],
      [0, 3, 6],
      [1, 4, 7],
      [2, 5, 8],
      [0, 4, 8],
      [2, 4, 6],
    ];
    for (let i = 0; i < lines.length; i++) {
      const [a, b, c] = lines[i]
      if (this.getMovedAtSquare(a)
          &&
          this.getMovedAtSquare(a) === this.getMovedAtSquare(b)
          &&
          this.getMovedAtSquare(a) === this.getMovedAtSquare(c))
      {
        return this.getMovedAtSquare(a)
      }
    }
    return null
  }

  getMovedAtSquare(index) {
    return this.board.getMovedAtSquare(index)
  }
}

export default Game
```

```javascript
// src/bluePrints/Board.js
import {BluePrint} from 'o-blue-print'
import Square from './Square'

class Board extends BluePrint {
  initialize() {
    this.allowAllMethodsAsCommands()

    this.squares = [0,1,2,3,4,5,6,7,8].map((i)=> 
      new Square({ parent: this, id: i }))
  }

  getRenderMethodName() {
    return 'renderBoard'
  }

  getStatus() {
    return `Next player: ${this.parent.getNextPlayer()}`
  }

  getSquareAtIndex(index) {
    return this.squares[index]
  }

  getMovedAtSquare(index) {
    return this.squares[index].getContent()
  }

  noMoreMovesLeft () {
    const emptySquares = this.getEmptySquares()
    return emptySquares.length <= 0
  }

  getEmptySquares() {
    return this.squares.filter((square) => square.isEmpty())
  }
}

export default Board
```

```javascript
// src/bluePrints/Square.js
import {BluePrint} from 'o-blue-print'

class Square extends BluePrint {
  initialize({id}) {
    this.allowAllMethodsAsCommands()

    this.id = id
    this.content = null
  }

  isEmpty() {
    return this.content === null
  }

  getId() {
    return this.id
  }

  getContent() {
    return this.content
  }

  playerMove() {
    if (this.content !== null) { return }
    const currentPlayer = this.parent.getNextPlayer()
    this.content = currentPlayer
    this.parent.shiftPlayer()
    this.invalidate()
  }
}

export default Square
```

## 15 - Update the status when the game finishes

```javascript
// src/bluePrints/Game.js
import {MainBluePrint} from 'o-blue-print'
import Board from './Board'

class Game extends MainBluePrint {
  initialize() {
    this.allowAllMethodsAsCommands()

    this.board = new Board({ parent: this })
    this.currentPlayer = 'X'
    this.gameStatus = { finished: false, winner: null }
  }

  getBoard() {
    return this.board
  }

  getNextPlayer() {
    return this.currentPlayer
  }

  isGameFinished() {
    return this.gameStatus.finished === true
  }

  getGameWinner() {
    return this.gameStatus.winner
  }

  shiftPlayer() {
    this.currentPlayer = this.getNextPlayer() === 'X' ? 'O' : 'X'
    this.updateGameStatus()
  }

  updateGameStatus() {
    const winner = this.getWinner()
    if (winner) {
      return this.endWithWinner(winner)
    }
    if (this.noMoreMovesLeft()) {
      return this.endWithDraw()
    }
  }

  noMoreMovesLeft() {
    return this.board.noMoreMovesLeft()
  }

  endWithDraw() {
    this.endWithWinner(null)
  }

  endWithWinner(winner) {
    this.gameStatus = { finished: true, winner }
    return
  }

  getWinner() {
    const lines = [
      [0, 1, 2],
      [3, 4, 5],
      [6, 7, 8],
      [0, 3, 6],
      [1, 4, 7],
      [2, 5, 8],
      [0, 4, 8],
      [2, 4, 6],
    ];
    for (let i = 0; i < lines.length; i++) {
      const [a, b, c] = lines[i]
      if (this.getMovedAtSquare(a)
          &&
          this.getMovedAtSquare(a) === this.getMovedAtSquare(b)
          &&
          this.getMovedAtSquare(a) === this.getMovedAtSquare(c))
      {
        return this.getMovedAtSquare(a)
      }
    }
    return null
  }

  getMovedAtSquare(index) {
    return this.board.getMovedAtSquare(index)
  }
}

export default Game
```

```javascript
// src/bluePrints/Board.js
import {BluePrint} from 'o-blue-print'
import Square from './Square'

class Board extends BluePrint {
  initialize() {
    this.allowAllMethodsAsCommands()

    this.squares = [0,1,2,3,4,5,6,7,8].map((i)=> 
      new Square({ parent: this, id: i }))
  }

  getRenderMethodName() {
    return 'renderBoard'
  }

  getStatus() {
    const isGameFinished = this.parent.isGameFinished()
    if (isGameFinished) {
      const winner = this.parent.getGameWinner()
      return winner ? `Winner ${winner} - Congrats!` : 'Draw'
    }   
    return `Next player: ${this.parent.getNextPlayer()}`
  }

  getSquareAtIndex(index) {
    return this.squares[index]
  }

  getMovedAtSquare(index) {
    return this.squares[index].getContent()
  }

  noMoreMovesLeft () {
    const emptySquares = this.getEmptySquares()
    return emptySquares.length <= 0
  }

  getEmptySquares() {
    return this.squares.filter((square) => square.isEmpty())
  }
}

export default Board
```

```javascript
// src/bluePrints/Square.js
import {BluePrint} from 'o-blue-print'

class Square extends BluePrint {
  initialize({id}) {
    this.allowAllMethodsAsCommands()

    this.id = id
    this.content = null
  }

  isEmpty() {
    return this.content === null
  }

  getId() {
    return this.id
  }

  getContent() {
    return this.content
  }

  playerMove() {
    if (this.content !== null || this.parent.isGameFinished()) { return }
    const currentPlayer = this.parent.getNextPlayer()
    this.content = currentPlayer
    this.parent.shiftPlayer()
    this.invalidate()
  }
}

export default Square
```

Note how the last modification to a `Component` class was done in #12 and all the complex logic of the game was added at the blue print level only.

## 17 - Add an application Router

At this point I will diverge a little from `React`s tutorial.

So far the application always showed the same component with different states.

I will modify the application to render a different screen when the game finishes.

Before going into the details of the solution I will implement it.

For starters instead of making the `<GameComponent>` to always render a `<BoardComponent>` I will make it to render different things depending on the current state of its Game blue print:


```javascript
// src/components/GameComponent.js
import React from 'react'
import ReactDOM from 'react-dom'
import BoardComponent from './BoardComponent'

class GameComponent extends React.Component {
  constructor (props) {
    super(props)
    this.invalidateComponent = this.invalidateComponent.bind(this)
  }

  componentDidMount () {
    const app = this.props.app
    app.on('invalidate', this.invalidateComponent)
  }

  componentWillUnmount () {
    const app = this.props.app
    app.removeListener('invalidate', this.invalidateComponent)
  }

  invalidateComponent () {
    this.forceUpdate()
  }

  render () {
    const app = this.props.app
    return (
      <div className="game">
        <div className="game-board">
          { app.getCurrentState().route() }   // <--- change
        </div>
        <div className="game-info">
          <div>{/* status */}</div>
          <ol>{/* TODO */}</ol>
        </div>
      </div>
    )
  }
}

export default GameComponent
```

But I haven't defined a router for the application yet.

Going back to the first file I will modify it to be

```javascript
// src/index.js
import React from 'react'
import ReactDOM from 'react-dom'
import GameComponent from './components/GameComponent'
import Game from './bluePrints/Game'
import GameRouter from './components/GameRouter'
import './index.css'

// Create the Game router
const gameRouter = new GameRouter()
// Create the Game
const game = new Game({ router: gameRouter })

const gameComponent = (
  <React.StrictMode>
    <GameComponent app={game} />
  </React.StrictMode>
)

ReactDOM.render(gameComponent, document.getElementById('root'))
```

I will create the GameRouter class


```javascript
// src/components/GameRouter.js
import React from 'react'
import ReactDOM from 'react-dom'
import { Router } from 'o-blue-print'
import BoardComponent from './BoardComponent'

class GameRouter extends Router {
  renderBoard(board) {
    return <BoardComponent app={board} />
  }
}

export default GameRouter
```

and lastly I will add the method 

```javascript
// src/bluePrints/Game.js
class Game extends MainBluePrint {
  // ...

  getCurrentState () {
    return this.board
  }

  // ...
```

If I run the application now it will run just like before but using a much more complex mechanism.

Before going on with the change I will explain the reasons for adding that much complexity.

### Finite State Machine pattern

Most applications can be thought as a machine that reacts to interactions.

At a given instant the machine is in a state `A`, a user does something like clicking a button and the machine passes to a state `B`.

To think a program or a part of a program like that is to think it using the `Finite State Machine` pattern (FSM for short).

FSM is a pattern where an object switches its own state.

React with BluePrint if particularly suited to model applications following the FSM pattern:

- the blue print is in a state `A`
- the application renders its `A` state during its render phase
- the user clicks a button
- the application changes its blue print to state `B`
- the blue print is in a state `B` and it triggers a new render phase
- the application render its `B` state during its render phase and it starts a new cycle all over again

The FSM pattern can be implemented in many different ways.

A possible implementation would be to make a `switch` statement depending on the state of the application:

```javascript
// src/components/BoardComponent.js
import React from 'react'
import ReactDOM from 'react-dom'
import SquareComponent from './SquareComponent'

class BoardComponent extends React.Component {
  render () {
    const app = this.props.app
    const appState = app.getState()
    return (
      <div className="game">
        <div className="game-board">
          { 
            switch(appState) {
              case 'playing':
                return <BoardComponent app={app.getBoard()} />
              case 'finished':
                return <FinishedGameComponent app={app.getFinishedGame()} />
            }
          }
        </div>
        <div className="game-info">
          <div>{/* status */}</div>
          <ol>{/* TODO */}</ol>
        </div>
      </div>
    )
  }
} of a program
```

This approach is good enough but it comes with some issues. Mainly the fact that every time a new state is added to the switch the code of a component that was already tested is modified, opening the possibility to introduce errors.

One of the guiding concepts of object oriented programming is to minimize the number of conditional statements like `if`s and `switch`s in a program.

Conditional statements make the logic of a program harder to understand, to test, to maintain and to extend. [Note 2]

For that reason object oriented programming uses a mechanism called `polymorphism` to replace conditional statements by regular method calls.

The object oriented approach to implement the FSM pattern is to have one object for each possible state and to switch objects when the state changes, removing conditional statements from the sender of the method

```javascript
  render () {
    const app = this.props.app
    return (
      <div className="game">
        <div className="game-board">
          { app.getCurrentState().route() }   // <--- change to FSM pattern
        </div>
        <div className="game-info">
          <div>{/* status */}</div>
          <ol>{/* TODO */}</ol>
        </div>
      </div>
    )
  }
```

Instead of asking its blue print for the state and use a conditional the Component says: whatever your current state is route yourself through the application router and render yourself.

Object oriented programming is mostly about objects trusting and delegating responsibilities to other objects to do a job.

## 18 - Render a different screen when the game finishes

I will render a different screen once the application finishes using the Router just added.

First I will modify Game to have two possible states, `Board` and `FinishedGame`


```javascript
// src/bluePrints/Game.js
import { MainBluePrint } from 'o-blue-print'
import Board from './Board'
import FinishedGame from './FinishedGame'

class Game extends MainBluePrint {
  initialize () {
    this.allowAllMethodsAsCommands()

    this.currentState = new Board({ parent: this })
    this.currentPlayer = 'X'
  }

  getCurrentState () {
    return this.currentState
  }

  getNextPlayer () {
    return this.currentPlayer
  }

  getGameWinner () {
    return this.currentState.getWinner()
  }

  shiftPlayer () {
    this.currentPlayer = this.getNextPlayer() === 'X' ? 'O' : 'X'
    this.updateGameStatus()
  }

  updateGameStatus () {
    const winner = this.getWinner()
    if (winner) {
      return this.endWithWinner(winner)
    }
    if (this.noMoreMovesLeft()) {
      return this.endWithDraw()
    }
  }

  noMoreMovesLeft () {
    return this.currentState.noMoreMovesLeft()
  }

  endWithDraw () {
    this.endWithWinner(null)
  }

  endWithWinner (winner) {
    this.currentState = new FinishedGame({ parent: this, winner})
  }

  getWinner () {
    const lines = [
      [0, 1, 2],
      [3, 4, 5],
      [6, 7, 8],
      [0, 3, 6],
      [1, 4, 7],
      [2, 5, 8],
      [0, 4, 8],
      [2, 4, 6]
    ]
    for (let i = 0; i < lines.length; i++) {
      const [a, b, c] = lines[i]
      if (this.getMovedAtSquare(a) &&
          this.getMovedAtSquare(a) === this.getMovedAtSquare(b) &&
          this.getMovedAtSquare(a) === this.getMovedAtSquare(c)) {
        return this.getMovedAtSquare(a)
      }
    }
    return null
  }

  getMovedAtSquare (index) {
    return this.currentState.getMovedAtSquare(index)
  }
}

export default Game
```

Now the Game starts in a `Board` state and when there are no more moves or a player wins it switches its state to `FinishedGame`.

Note how the `Game` delegates many of its previous methods to its current state. That is a key part of the FSM pattern implementation.

The code of the new class `FinishedGame` is as follows


```javascript
// src/bluePrints/FinishedGame.js
import { BluePrint } from 'o-blue-print'

class FinishedGame extends BluePrint {
  initialize ({winner}) {
    this.allowAllMethodsAsCommands()

    this.winner = winner
  }

  getRenderMethodName() {
    return 'renderFinishedGame'
  }

  isDraw() {
    return this.winner === null
  }

  getWinner() {
    return this.winner
  }

  startNewGame() {
    this.parent.newGame()
    this.invalidate()
  }
}

export default FinishedGame
```

If I run the application now and the game finishes I will get the error

```
Error: The class GameRouter must implement the method 'renderFinishedGame(bluePrint)'
```

I will implement that method and the `FinishedGameComponent`:


```javascript
// src/components/GameRouter.js
import React from 'react'
import ReactDOM from 'react-dom'
import { Router } from 'o-blue-print'
import BoardComponent from './BoardComponent'
import FinishedGameComponent from './FinishedGameComponent'

class GameRouter extends Router {
  renderBoard(board) {
    return <BoardComponent app={board} />
  }

  renderFinishedGame(finishedGame) {
    return <FinishedGameComponent app={finishedGame} />
  }
}

export default GameRouter
```

```javascript
// src/components/FinishedGameComponent.js
import React from 'react'
import ReactDOM from 'react-dom'

class FinishedGameComponent extends React.Component {
  render () {
    const app = this.props.app
    const message = app.isDraw() ? 'It was a draw' : `The winner was ${app.getWinner()}`

    return (
      <React.Fragment>
        <div>The game finished</div>
        <div>{message}</div>
        <div>Want to play another one?</div>
        <div>
          <button onClick={()=>app.startNewGame()}>Play another one</button>
        </div>
      </React.Fragment>
    )
  }
}

export default FinishedGameComponent
```

A last missing step is to add the method `Game.newGame` and cleanup some methods that are no longer called

```javascript
// src/bluePrints/Game.js
import { MainBluePrint } from 'o-blue-print'
import Board from './Board'
import FinishedGame from './FinishedGame'

class Game extends MainBluePrint {
  initialize () {
    this.allowAllMethodsAsCommands()

    this.currentState = null
    this.currentPlayer = null
    this.newGame()
  }

  newGame() {
    this.currentState = new Board({ parent: this })
    this.currentPlayer = 'X'
  }

  getCurrentState () {
    return this.currentState
  }

  getNextPlayer () {
    return this.currentPlayer
  }

  getGameWinner () {
    return this.currentState.getWinner()
  }

  shiftPlayer () {
    this.currentPlayer = this.getNextPlayer() === 'X' ? 'O' : 'X'
    this.updateGameStatus()
  }

  updateGameStatus () {
    const winner = this.getWinner()
    if (winner) {
      return this.endWithWinner(winner)
    }
    if (this.noMoreMovesLeft()) {
      return this.endWithDraw()
    }
  }

  noMoreMovesLeft () {
    return this.currentState.noMoreMovesLeft()
  }

  endWithDraw () {
    this.endWithWinner(null)
  }

  endWithWinner (winner) {
    this.currentState = new FinishedGame({ parent: this, winner})
  }

  getWinner () {
    const lines = [
      [0, 1, 2],
      [3, 4, 5],
      [6, 7, 8],
      [0, 3, 6],
      [1, 4, 7],
      [2, 5, 8],
      [0, 4, 8],
      [2, 4, 6]
    ]
    for (let i = 0; i < lines.length; i++) {
      const [a, b, c] = lines[i]
      if (this.getMovedAtSquare(a) &&
          this.getMovedAtSquare(a) === this.getMovedAtSquare(b) &&
          this.getMovedAtSquare(a) === this.getMovedAtSquare(c)) {
        return this.getMovedAtSquare(a)
      }
    }
    return null
  }

  getMovedAtSquare (index) {
    return this.currentState.getMovedAtSquare(index)
  }

  isGameFinished() {
    return false
  }
}

export default Game
```

## 19 - Keep the history of the moves

Going back the `React` tutorial I will make the application to keep the history of the game moves.

To achieve that before changing the state on each move I will make a copy of the whole game and keep it in an array.

But first I will make a small change in the method `Square.playerMove()`

Currently the move is made in the class `Square`.

That worked pretty well but it's not the only possible implementation.

A different approach might by to be the `Game` the one to make the move.

The difference is that by moving that responsibility up to the `Game` it would be more simple for anybody wanting to make a move to ask for it to the game instead of the square:

```javascript
game.makeNextPlayerMoveAt({square: 1})
```

instead of

```javascript
game.getBoard().getSquare(1).playerMove()
```

The choice of the `Game` object instead of the `Board` object is to make `Game` the main interface for all modifications in the game. That would make it easier to trace and debug changes since `Game` would be the only entry point.

I will modify the Square class

```javascript
// src/bluePrints/Square.js
import { BluePrint } from 'o-blue-print'

class Square extends BluePrint {
  initialize ({ id }) {
    this.allowAllMethodsAsCommands()

    this.id = id
    this.content = null
  }

  isEmpty () {
    return this.content === null
  }

  getId () {
    return this.id
  }

  getContent () {
    return this.content
  }

  playerMove () {
    this.parent.makeNextPlayerMoveAt(this.id)
  }

  fillWith(content) {
    this.content = content
    this.invalidate()
  }
}

export default Square
```

the `Game` class

```javascript
// src/bluePrints/Game.js
import { MainBluePrint } from 'o-blue-print'
import Board from './Board'
import FinishedGame from './FinishedGame'

class Game extends MainBluePrint {
  initialize () {
    this.allowAllMethodsAsCommands()

    this.currentState = null
    this.currentPlayer = null
    this.newGame()
  }

  newGame () {
    this.currentState = new Board({ parent: this })
    this.currentPlayer = 'X'
  }

  getCurrentState () {
    return this.currentState
  }

  getNextPlayer () {
    return this.currentPlayer
  }

  getGameWinner () {
    return this.currentState.getWinner()
  }

  shiftPlayer () {
    this.currentPlayer = this.getNextPlayer() === 'X' ? 'O' : 'X'
    this.updateGameStatus()
  }

  updateGameStatus () {
    const winner = this.getWinner()
    if (winner) {
      return this.endWithWinner(winner)
    }
    if (this.noMoreMovesLeft()) {
      return this.endWithDraw()
    }
  }

  noMoreMovesLeft () {
    return this.currentState.noMoreMovesLeft()
  }

  endWithDraw () {
    this.endWithWinner(null)
  }

  endWithWinner (winner) {
    this.currentState = new FinishedGame({ parent: this, winner })
  }

  getWinner () {
    const lines = [
      [0, 1, 2],
      [3, 4, 5],
      [6, 7, 8],
      [0, 3, 6],
      [1, 4, 7],
      [2, 5, 8],
      [0, 4, 8],
      [2, 4, 6]
    ]
    for (let i = 0; i < lines.length; i++) {
      const [a, b, c] = lines[i]
      if (this.getMovedAtSquare(a) &&
          this.getMovedAtSquare(a) === this.getMovedAtSquare(b) &&
          this.getMovedAtSquare(a) === this.getMovedAtSquare(c)) {
        return this.getMovedAtSquare(a)
      }
    }
    return null
  }

  getMovedAtSquare (index) {
    return this.currentState.getMovedAtSquare(index)
  }

  makeNextPlayerMoveAt(squareIndex) {
    if (!this.currentState.emptySquare(squareIndex)) { return }
    const currentPlayer = this.getNextPlayer()
    this.currentState.fillSquare({player: currentPlayer, square: squareIndex})
    this.shiftPlayer()
    this.invalidate()
  }
}

export default Game
```

and the `Board` class

```javascript
// src/bluePrints/Board.js
import { BluePrint } from 'o-blue-print'
import Square from './Square'

class Board extends BluePrint {
  initialize () {
    this.allowAllMethodsAsCommands()

    this.squares = [0, 1, 2, 3, 4, 5, 6, 7, 8].map((i) =>
      new Square({ parent: this, id: i }))
  }

  getRenderMethodName() {
    return 'renderBoard'
  }

  getStatus () {
    return `Next player: ${this.parent.getNextPlayer()}`
  }

  getSquareAtIndex (index) {
    return this.squares[index]
  }

  getMovedAtSquare (index) {
    return this.squares[index].getContent()
  }

  noMoreMovesLeft () {
    const emptySquares = this.getEmptySquares()
    return emptySquares.length <= 0
  }

  getEmptySquares () {
    return this.squares.filter((square) => square.isEmpty())
  }

  emptySquare(squareIndex) {
    return this.squares[squareIndex].isEmpty()    
  }

  fillSquare({player, square}) {
    this.squares[square].fillWith(player)
  }
}

export default Board
```

Now I will make the `Game` to keep a copy of the current state before making the next move.

To do that I will implement a `serialize()` method on each blue print.

By making each blue print to serialize and restore a copy of itself blue prints can be as complex or simple as needed without other objects needing to be aware of that.

```javascript
// src/bluePrints/Game.js
import { MainBluePrint } from 'o-blue-print'
import Board from './Board'
import FinishedGame from './FinishedGame'

class Game extends MainBluePrint {
  initialize () {
    this.allowAllMethodsAsCommands()

    this.currentState = null
    this.currentPlayer = null
    this.gameHistory = []
    this.newGame()
  }

  newGame () {
    this.currentState = new Board({ parent: this })
    this.currentPlayer = 'X'
    this.gameHistory = []
    this.storeGameState()
  }

  getCurrentState () {
    return this.currentState
  }

  getNextPlayer () {
    return this.currentPlayer
  }

  getGameWinner () {
    return this.currentState.getWinner()
  }

  shiftPlayer () {
    this.currentPlayer = this.getNextPlayer() === 'X' ? 'O' : 'X'
  }

  updateGameStatus () {
    const winner = this.getWinner()
    if (winner) {
      return this.endWithWinner(winner)
    }
    if (this.noMoreMovesLeft()) {
      return this.endWithDraw()
    }
  }

  noMoreMovesLeft () {
    return this.currentState.noMoreMovesLeft()
  }

  endWithDraw () {
    this.endWithWinner(null)
  }

  endWithWinner (winner) {
    this.currentState = new FinishedGame({ parent: this, winner })
  }

  getWinner () {
    const lines = [
      [0, 1, 2],
      [3, 4, 5],
      [6, 7, 8],
      [0, 3, 6],
      [1, 4, 7],
      [2, 5, 8],
      [0, 4, 8],
      [2, 4, 6]
    ]
    for (let i = 0; i < lines.length; i++) {
      const [a, b, c] = lines[i]
      if (this.getMovedAtSquare(a) &&
          this.getMovedAtSquare(a) === this.getMovedAtSquare(b) &&
          this.getMovedAtSquare(a) === this.getMovedAtSquare(c)) {
        return this.getMovedAtSquare(a)
      }
    }
    return null
  }

  getMovedAtSquare (index) {
    return this.currentState.getMovedAtSquare(index)
  }

  makeNextPlayerMoveAt(squareIndex) {
    if (!this.currentState.emptySquare(squareIndex)) { return }
    const currentPlayer = this.getNextPlayer()
    this.currentState.fillSquare({player: currentPlayer, square: squareIndex})
    this.shiftPlayer()
    this.storeGameState()
    this.updateGameStatus()
    this.invalidate()
  }

  storeGameState() {
    const gameStateCopy = this.makeGameStateCopy()
    this.gameHistory.push(gameStateCopy)
  }

  makeGameStateCopy() {
    return {
      currentState: this.currentState.serialize(),
      currentPlayer: this.currentPlayer,
    }
  }
}

export default Game
```

```javascript
// src/bluePrints/Board.js
import { BluePrint } from 'o-blue-print'
import Square from './Square'

class Board extends BluePrint {
  initialize () {
    this.allowAllMethodsAsCommands()

    this.squares = [0, 1, 2, 3, 4, 5, 6, 7, 8].map((i) =>
      new Square({ parent: this, id: i }))
  }

  getRenderMethodName() {
    return 'renderBoard'
  }

  getStatus () {
    return `Next player: ${this.parent.getNextPlayer()}`
  }

  getSquareAtIndex (index) {
    return this.squares[index]
  }

  getMovedAtSquare (index) {
    return this.squares[index].getContent()
  }

  noMoreMovesLeft () {
    const emptySquares = this.getEmptySquares()
    return emptySquares.length <= 0
  }

  getEmptySquares () {
    return this.squares.filter((square) => square.isEmpty())
  }

  emptySquare(squareIndex) {
    return this.squares[squareIndex].isEmpty()    
  }

  fillSquare({player, square}) {
    this.squares[square].fillWith(player)
  }

  serialize() {
    return {
      squares: this.squares.map((square)=>square.serialize())
    }
  }
}

export default Board
```

```javascript
// src/bluePrints/Square.js
import { BluePrint } from 'o-blue-print'

class Square extends BluePrint {
  initialize ({ id }) {
    this.allowAllMethodsAsCommands()

    this.id = id
    this.content = null
  }

  isEmpty () {
    return this.content === null
  }

  getId () {
    return this.id
  }

  getContent () {
    return this.content
  }

  playerMove () {
    this.parent.makeNextPlayerMoveAt(this.id)
  }

  fillWith(content) {
    this.content = content
    this.invalidate()
  }

  serialize() {
    return {id: this.id, content: this.content}
  }
}

export default Square
```

## 20 - Show the game moves history

The next step is to show the game history to the user so she can restore a previous move.

There are different ways to implement the moves history. I will pick a simple one to just make it work.


```javascript
// src/components/GameComponent.js
/* eslint-disable no-unused-vars */
import React from 'react'
import ReactDOM from 'react-dom'
import BoardComponent from './BoardComponent'

class GameComponent extends React.Component {
  constructor (props) {
    super(props)
    this.invalidateComponent = this.invalidateComponent.bind(this)
  }

  componentDidMount () {
    const app = this.props.app
    app.on('invalidate', this.invalidateComponent)
  }

  componentWillUnmount () {
    const app = this.props.app
    app.removeListener('invalidate', this.invalidateComponent)
  }

  invalidateComponent () {
    this.forceUpdate()
  }

  render () {
    const app = this.props.app
    return (
      <div className="game">
        <div className="game-board">
          { app.getCurrentState().route() }
        </div>
        <div className="game-info">
          <div>Game history</div>
          <ol>{
            app.getGameHistory().map((move, index)=>(
              <div key={index}>
                <a onClick={()=>app.gotoMove(index)}>{`Move ${index}`}</a>
              </div>
            ))
          }</ol>
        </div>
      </div>
    )
  }
}

export default GameComponent
```

```javascript
// src/bluePrints/Game.js
import { MainBluePrint } from 'o-blue-print'
import Board from './Board'
import FinishedGame from './FinishedGame'

class Game extends MainBluePrint {
  initialize () {
    this.allowAllMethodsAsCommands()

    this.currentState = null
    this.currentPlayer = null
    this.gameHistory = []
    this.newGame()
  }

  newGame () {
    this.currentState = new Board({ parent: this })
    this.currentPlayer = 'X'
    this.gameHistory = []
    this.storeGameState()
  }

  getCurrentState () {
    return this.currentState
  }

  getNextPlayer () {
    return this.currentPlayer
  }

  getGameWinner () {
    return this.currentState.getWinner()
  }

  getGameHistory () {
    return this.gameHistory
  }

  shiftPlayer () {
    this.currentPlayer = this.getNextPlayer() === 'X' ? 'O' : 'X'
  }

  updateGameStatus () {
    const winner = this.getWinner()
    if (winner) {
      return this.endWithWinner(winner)
    }
    if (this.noMoreMovesLeft()) {
      return this.endWithDraw()
    }
  }

  noMoreMovesLeft () {
    return this.currentState.noMoreMovesLeft()
  }

  endWithDraw () {
    this.endWithWinner(null)
  }

  endWithWinner (winner) {
    this.currentState = new FinishedGame({ parent: this, winner })
  }

  getWinner () {
    const lines = [
      [0, 1, 2],
      [3, 4, 5],
      [6, 7, 8],
      [0, 3, 6],
      [1, 4, 7],
      [2, 5, 8],
      [0, 4, 8],
      [2, 4, 6]
    ]
    for (let i = 0; i < lines.length; i++) {
      const [a, b, c] = lines[i]
      if (this.getMovedAtSquare(a) &&
          this.getMovedAtSquare(a) === this.getMovedAtSquare(b) &&
          this.getMovedAtSquare(a) === this.getMovedAtSquare(c)) {
        return this.getMovedAtSquare(a)
      }
    }
    return null
  }

  getMovedAtSquare (index) {
    return this.currentState.getMovedAtSquare(index)
  }

  makeNextPlayerMoveAt (squareIndex) {
    if (!this.currentState.emptySquare(squareIndex)) { return }
    const currentPlayer = this.getNextPlayer()
    this.currentState.fillSquare({ player: currentPlayer, square: squareIndex })
    this.shiftPlayer()
    this.storeGameState()
    this.updateGameStatus()
    this.invalidate()
  }

  storeGameState () {
    const gameStateCopy = this.makeGameStateCopy()
    this.gameHistory.push(gameStateCopy)
  }

  makeGameStateCopy () {
    return {
      moveIndex: this.gameHistory.length,
      currentState: this.currentState.serialize(),
      currentPlayer: this.currentPlayer
    }
  }

  gotoMove (moveIndex) {
    const gameState = this.gameHistory[moveIndex]
    this.restoreFrom(gameState)
    this.gameHistory = this.gameHistory.slice(0, moveIndex+1)
  }

  _gotoMove (moveIndex) {
    const gameState = this.gameHistory[moveIndex]
    this.restoreFrom(gameState)
    this.gameHistory = this.gameHistory.slice(0, moveIndex)
  }

  restoreFrom (gameState) {
    this.currentState = new Board({ parent: this })
    this.currentState.restoreFrom(gameState.currentState)
    this.currentPlayer = gameState.currentPlayer
    this.invalidate()
  }
}

export default Game
```

```javascript
// src/bluePrints/Board.js
import { BluePrint } from 'o-blue-print'
import Square from './Square'

class Board extends BluePrint {
  initialize () {
    this.allowAllMethodsAsCommands()

    this.squares = [0, 1, 2, 3, 4, 5, 6, 7, 8].map((i) =>
      new Square({ parent: this, id: i }))
  }

  getRenderMethodName() {
    return 'renderBoard'
  }

  getStatus () {
    return `Next player: ${this.parent.getNextPlayer()}`
  }

  getSquareAtIndex (index) {
    return this.squares[index]
  }

  getMovedAtSquare (index) {
    return this.squares[index].getContent()
  }

  noMoreMovesLeft () {
    const emptySquares = this.getEmptySquares()
    return emptySquares.length <= 0
  }

  getEmptySquares () {
    return this.squares.filter((square) => square.isEmpty())
  }

  emptySquare(squareIndex) {
    return this.squares[squareIndex].isEmpty()    
  }

  fillSquare({player, square}) {
    this.squares[square].fillWith(player)
  }

  serialize() {
    return {
      squares: this.squares.map((square)=>square.serialize())
    }
  }

  restoreFrom(boardState) {
    this.squares = boardState.squares.map((squareState) => {
      const square = new Square({ parent: this })
      square.restoreFrom(squareState)
      return square
    })
  }
}

export default Board
```

```javascript
// src/bluePrints/Square.js
import { BluePrint } from 'o-blue-print'

class Square extends BluePrint {
  initialize ({ id }) {
    this.allowAllMethodsAsCommands()

    this.id = id
    this.content = null
  }

  isEmpty () {
    return this.content === null
  }

  getId () {
    return this.id
  }

  getContent () {
    return this.content
  }

  playerMove () {
    this.parent.makeNextPlayerMoveAt(this.id)
  }

  fillWith(content) {
    this.content = content
    this.invalidate()
  }

  serialize() {
    return {id: this.id, content: this.content}
  }

  restoreFrom(squareState) {
    this.id = squareState.id
    this.content = squareState.content
  }
}

export default Square
```

## 21 - Support back button

In the previous step the application navigates to a past move without using the browsers url.

I will change the implementation to use url navigation.

First I will make the root <Component> to notify the `Game` blue print when the browser url changes.

```javascript
// src/components/GameComponent.js
/* eslint-disable no-unused-vars */
import React from 'react'
import ReactDOM from 'react-dom'
import BoardComponent from './BoardComponent'

class GameComponent extends React.Component {
  constructor (props) {
    super(props)
    this.invalidateComponent = this.invalidateComponent.bind(this)
    this.syncUrl = this.syncUrl.bind(this)
  }

  componentDidMount () {
    const app = this.props.app
    app.on('invalidate', this.invalidateComponent)
    window.addEventListener('popstate', this.syncUrl)
  }

  componentWillUnmount () {
    const app = this.props.app
    app.removeListener('invalidate', this.invalidateComponent)
    window.removeEventListener('popstate', this.syncUrl)
  }

  invalidateComponent () {
    this.forceUpdate()
  }

  syncUrl() {
    const app = this.props.app
    app.syncFromUrl()
  }

  render () {
    const app = this.props.app
    return (
      <div className="game">
        <div className="game-board">
          { app.getCurrentState().route() }
        </div>
        <div className="game-info">
          <div>Game history</div>
          <ol>{
            app.getGameHistory().map((move, index) => (
              <div>
                <a onClick={() => app.loadMoveUrl(index)}>{`Move ${index}`}</a>
              </div>
            ))
          }</ol>
        </div>
      </div>
    )
  }
}

export default GameComponent
```

Then I will add the method `syncFromUrl` to `Game` and I will save a new url every time a move is made by calling `this.saveUrl`.

Finally when the user clicks the links from the `Game history` instead of refreshing the state directly I will redirect the user to the move link


```javascript
// src/bluePrints/Game.js
import { MainBluePrint } from 'o-blue-print'
import Board from './Board'
import FinishedGame from './FinishedGame'

class Game extends MainBluePrint {
  initialize () {
    this.allowAllMethodsAsCommands()

    this.currentState = null
    this.currentPlayer = null
    this.gameHistory = []
    this.newGame()
  }

  newGame () {
    this.currentState = new Board({ parent: this })
    this.currentPlayer = 'X'
    this.gameHistory = []
    this.storeGameState()
  }

  syncFromUrl () {
    const path = this.getCurrentPath()
    const match = path.match(/[/]move[/](\d+)$/)
    if (!match) return
    const moveIndex = parseInt(match[1])
    this.gotoMove(moveIndex)
  }

  getCurrentState () {
    return this.currentState
  }

  getNextPlayer () {
    return this.currentPlayer
  }

  getGameWinner () {
    return this.currentState.getWinner()
  }

  getGameHistory () {
    return this.gameHistory
  }

  shiftPlayer () {
    this.currentPlayer = this.getNextPlayer() === 'X' ? 'O' : 'X'
  }

  updateGameStatus () {
    const winner = this.getWinner()
    if (winner) {
      return this.endWithWinner(winner)
    }
    if (this.noMoreMovesLeft()) {
      return this.endWithDraw()
    }
  }

  noMoreMovesLeft () {
    return this.currentState.noMoreMovesLeft()
  }

  endWithDraw () {
    this.endWithWinner(null)
  }

  endWithWinner (winner) {
    this.currentState = new FinishedGame({ parent: this, winner })
  }

  getWinner () {
    const lines = [
      [0, 1, 2],
      [3, 4, 5],
      [6, 7, 8],
      [0, 3, 6],
      [1, 4, 7],
      [2, 5, 8],
      [0, 4, 8],
      [2, 4, 6]
    ]
    for (let i = 0; i < lines.length; i++) {
      const [a, b, c] = lines[i]
      if (this.getMovedAtSquare(a) &&
          this.getMovedAtSquare(a) === this.getMovedAtSquare(b) &&
          this.getMovedAtSquare(a) === this.getMovedAtSquare(c)) {
        return this.getMovedAtSquare(a)
      }
    }
    return null
  }

  getMovedAtSquare (index) {
    return this.currentState.getMovedAtSquare(index)
  }

  makeNextPlayerMoveAt (squareIndex) {
    if (!this.currentState.emptySquare(squareIndex)) { return }
    const currentPlayer = this.getNextPlayer()
    this.currentState.fillSquare({ player: currentPlayer, square: squareIndex })
    this.shiftPlayer()
    this.storeGameState()
    this.updateGameStatus()
    this.invalidate()
  }

  storeGameState () {
    const gameStateCopy = this.makeGameStateCopy()
    this.gameHistory.push(gameStateCopy)
    this.saveUrl(`/move/${this.gameHistory.length-1}`, gameStateCopy)
  }

  makeGameStateCopy () {
    return {
      moveIndex: this.gameHistory.length,
      currentState: this.currentState.serialize(),
      currentPlayer: this.currentPlayer
    }
  }

  loadMoveUrl(index) {
    for(let i = index; i < this.gameHistory.length-1; i++) {
      window.history.back()
    }
    this.invalidate()
  }

  gotoMove (moveIndex) {
    const gameState = this.gameHistory[moveIndex]
    if (!gameState) { return }
    this.restoreFrom(gameState)
    this.gameHistory = this.gameHistory.slice(0, moveIndex+1)
  }

  restoreFrom (gameState) {
    this.currentState = new Board({ parent: this })
    this.currentState.restoreFrom(gameState.currentState)
    this.currentPlayer = gameState.currentPlayer
    this.invalidate()
  }
}

export default Game
```

and that's it.

To allow to navigate from a url directly I would need to use some kind of persistency like the browser local storage but I won't cover that in this tutorial.

Hope `React` with `BluePrint` makes writing applications more enjoyable.


#### [Note 1]

The use of the `Observer` pattern is not always the best design choice but it does have some advantages.

One is that it doesn't force the observer to comply with a given interface.

If the BluePrint would expect its observers to have a method `.render()` to call it would force objects other than `Component`s to implement a `render()` method or to use an `Adaptor`. And BluePrint observers may not always be `Component` objects.

#### [Note 2]

There's always a tension and a trade off between taking object oriented principles to its ends and being pragmatic.

A solution so abstract that it makes it harder to understand and maintain may not be a better solution than a less abstract one.

When should an `if` statement be replaced by a design pattern? When the `if` statement becomes hard to maintain. But the other way around stands true as well.

Any solution should be dropped the moment that it creates more problems and annoyances than its trying to solve. Abstract object oriented patterns too.