const EventEmitter = require('events')
const BluePrintCommon = require('./BluePrintCommon')
const BrowserDevice = require('./BrowserDevice')

class MainBluePrint extends EventEmitter {
  constructor (params) {
    super()

    this.isInvalidated = false
    this.router = params ? params.router : undefined
    this.lastUrl = ''
    this.publicCommands = [
      'aboutToRender',
      'evaluate',
      'command',
      'getCurrentUrl',
      'getRouter',
      'gotoUrl',
      'saveUrl',
      'setUrl',
      'invalidate',
      'invalidateNow',
      'on',
      'respondsTo',
      'route',
      'routeWithMethod',
      'syncFromUrl',
      'getUUID'
    ]
    this.device = null

    this.useBrowserDevice()

    this.initialize(params)
    return this.asReference()
  }

  useBrowserDevice () {
    const browserDevice = new BrowserDevice()
    this.setDevice(browserDevice)
  }

  setDevice (device) {
    this.device = device
  }

  getClassName () {
    return this.constructor.name
  }

  getRouter () {
    return this.router
  }

  route () {
    return this.getRouter().route(this)
  }

  routeWithMethod (methodName) {
    return this.getRouter().routeWithMethod(methodName, this)
  }

  getCurrentPath () {
    return this.device.getCurrentPath()
  }

  aboutToRender () {
    if (this.urlHasChanged() && this.device.supportsUrlRouting()) {
      this.syncFromUrl()
      this.lastUrl = this.getCurrentPath()
    }
  }

  urlHasChanged () {
    return this.lastUrl !== this.getCurrentPath()
  }

  evaluate ({ command, params }) {
    return this.acceptsCommand(command)
      ? Reflect.apply(this[command], this, params)
      : this.raiseCommandNotFoundError({ command, params })
  }

  respondsTo (command) {
    return this.acceptsCommand(command)
  }

  acceptsCommand (command) {
    return this.publicCommands.includes(command)
  }

  /**
   * Re-render the whole MainBluePrint tree. Queue the rendering for the next tick to avoid locking
   * the calling component during the render.
   * React takes care of efficiently deciding whether an actual rendering is required for each
   * child component.
   * It would also possible to implement several custom optimizations to avoid innecesary
   * renders but currently there is none.
   * This is the only place where a re-render occurs.
   * At this point either the blue print has its new state to update the components or it
   * started async calls to get a new state.
   * When the new state arrives from the async calls it will trigger a new invalidate
   * which in turn will re-render the components.
   * In any case the async code occurs only within BluePrint objects, the render phase
   * always sync and no component needs to handle state, just read only props.
   */
  invalidate () {
    // if this phase is already invalidated do not queue a new invalidation event
    if (this.isInvalidated === true) { return }
    this.isInvalidated = true
    process.nextTick(() => this.invalidateNow())
  }

  invalidateNow () {
    // reset the isInvalidated flag **before** triggering the event
    // the render phase should not invalidate the BluePrint but it might
    this.isInvalidated = false
    this.emit('invalidate')
  }

  saveUrl (url, state = {}) {
    this.device.saveUrl({ url, state })
  }

  setUrl (url) {
    this.device.setUrl(url)
  }

  gotoUrl (url, state) {
    this.device.gotoUrl(url)
  }
}

BluePrintCommon.attachTo(MainBluePrint)

module.exports = MainBluePrint
